use anyhow::Result;
use std::{
    fs::File,
    io::{Read, Seek, SeekFrom},
};

pub struct RMFrames {
    file: File,
    start: u64,
    cursor: usize,
    framebuffer_size: usize,
}

impl RMFrames {
    pub fn init(path: &str, offset: usize, framebuffer_size: usize) -> Result<Self> {
        let start = offset as u64;
        let cursor = 0;
        let file = File::open(path)?;

        let mut streamer = RMFrames {
            file,
            start,
            cursor,
            framebuffer_size,
        };
        streamer.next_frame()?;
        Ok(streamer)
    }

    pub fn next_frame(&mut self) -> std::io::Result<()> {
        self.file.seek(SeekFrom::Start(self.start))?;
        self.cursor = 0;
        Ok(())
    }
}

impl Read for RMFrames {
    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
        let requested = buf.len();
        let bytes_read = if self.cursor + requested < self.framebuffer_size {
            self.file.read(buf)?
        } else {
            let rest = self.framebuffer_size - self.cursor;
            self.file.read(&mut buf[0..rest])?
        };

        self.cursor += bytes_read;
        if self.cursor == self.framebuffer_size {
            self.next_frame()?;
        }

        Ok(bytes_read)
    }
}
